@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 
 # NOTE:
 # Versions 1.9 (or higher) of aclocal and automake are required.
@@ -29,6 +29,11 @@ then
     AUTOMAKE=`which automake`
 fi
 
+if [ -z "$AUTORECONF" ]
+then
+    AUTORECONF=`which autoreconf`
+fi
+
 if [ -z "$AUTOCONF" ]
 then
     AUTOCONF=`which autoconf`
@@ -44,15 +49,36 @@ then
     fi
 fi
 
+if [ ! -d m4 ] ;
+then
+mkdir m4
+fi
 
-echo "Calling $LIBTOOLIZE $force"
-$LIBTOOLIZE $force || die "libtoolize failed"
+echo "Calling $AUTORECONF"
+$AUTORECONF
 
-echo "Calling $ACLOCAL..."
-$ACLOCAL -I m4 || die "aclocal failed"
+ret=$?
 
-echo "Calling $AUTOCONF..."
-$AUTOCONF || die "autoconf failed"
+if [ $ret -ne 0 ] ; then
+echo "autoreconf FAILED"
+echo "trying '$LIBTOOLIZE --force; $AUTOMAKE --add-missing ; $AUTORECONF'"
+$LIBTOOLIZE --force
+$AUTOMAKE  --add-missing
+$AUTORECONF
+if [ ! -e config.guess ] ; then 
+$AUTOMAKE  --add-missing
+$AUTORECONF
+fi
+fi
 
-echo "Calling $AUTOMAKE --add-missing..."
-$AUTOMAKE --add-missing || die "automake failed"
+#echo "Calling $LIBTOOLIZE $force"
+#$LIBTOOLIZE $force || die "libtoolize failed"
+
+#echo "Calling $ACLOCAL..."
+#$ACLOCAL -I m4 || die "aclocal failed"
+
+#echo "Calling $AUTOCONF..."
+#$AUTOCONF || die "autoconf failed"
+
+#echo "Calling $AUTOMAKE --add-missing..."
+#$AUTOMAKE --add-missing || die "automake failed"
