@@ -39,35 +39,33 @@
 
 #include	"cmd.h"
 
+#ifdef	NEEDSTRDUP
+char	*strdup(const char *s);
+#endif
 
-#define	FALSE	0
-#define	TRUE	1
+#define	LINSIZ		10240
 
-static Enum_T	BoolEnum[] = {
+
+static Bool_T	BoolEnum[] = {
 	{	(char*)"FALSE",	FALSE},
 	{	(char*)"TRUE",   TRUE},
 	{	(char*)"false",	FALSE},
 	{	(char*)"true",   TRUE},
 	{	(char*)"0",	    FALSE},
 	{	(char*)"1",	     TRUE},
-  { (char*)"NO",    FALSE},
-  { (char*)"YES",    TRUE},
-  { (char*)"No",    FALSE},
-  { (char*)"Yes",    TRUE},
-  { (char*)"no",    FALSE},
-  { (char*)"yes",    TRUE},
-  { (char*)"N",     FALSE},
-  { (char*)"Y",      TRUE},
-  { (char*)"n",     FALSE},
-  { (char*)"y",      TRUE},
-	{	0,		0	}
+	{ (char*)"NO",    FALSE},
+	{ (char*)"YES",    TRUE},
+	{ (char*)"No",    FALSE},
+	{ (char*)"Yes",    TRUE},
+	{ (char*)"no",    FALSE},
+	{ (char*)"yes",    TRUE},
+	{ (char*)"N",     FALSE},
+	{ (char*)"Y",      TRUE},
+	{ (char*)"n",     FALSE},
+	{ (char*)"y",      TRUE},
+	END_ENUM
 };
 
-#ifdef	NEEDSTRDUP
-char	*strdup(const char *s);
-#endif
-
-#define	LINSIZ		10240
 
 static char
 *GetLine(FILE	*fp,
@@ -89,6 +87,8 @@ SetParam(Cmd_T	*cmd,
 				 char	*s),
 SetEnum(Cmd_T	*cmd,
 				char	*s),
+SetBool(Cmd_T	*cmd,
+				char	*s),
 SetFlag(Cmd_T	*cmd,
 				char	*s),
 SetSubrange(Cmd_T	*cmd,
@@ -104,6 +104,8 @@ SetLte(Cmd_T	*cmd,
 CmdError(char	*opt),
 EnumError(Cmd_T	*cmd,
 					char	*s),
+BoolError(Cmd_T	*cmd,
+					char	*s),
 SubrangeError(Cmd_T	*cmd,
 							int	n),
 GteError(Cmd_T	*cmd,
@@ -123,6 +125,10 @@ PrintEnum(Cmd_T	*cmd,
 					int	TypeFlag,
 					int	ValFlag,
 					FILE	*fp),
+PrintBool(Cmd_T	*cmd,
+					int	TypeFlag,
+					int	ValFlag,
+					FILE	*fp),
 PrintFlag(Cmd_T	*cmd,
 					int	TypeFlag,
 					int	ValFlag,
@@ -177,10 +183,7 @@ GetParams(int	*n,
 	char	**argv = *a,
 	*s,
 	*p,
-	*CmdFile,
 	*defCmd;
-	FILE	*fp;
-	int	IsPipe;
 	
 #if defined(MSDOS)||defined(_WIN32)
 	char	*dot = 0;
@@ -190,10 +193,10 @@ GetParams(int	*n,
 	if(!(Line=malloc(LINSIZ))) {
 		fprintf(stderr, "GetParams(): Unable to alloc %d bytes\n",
 						LINSIZ);
-		exit(1);
+		exit(IRSTLM_CMD_ERROR_MEMORY);
 	}
 	for(ProgName=*argv+strlen(*argv);
-	    ProgName-->*argv && *ProgName!='/' && *ProgName!='\\';);
+			ProgName-->*argv && *ProgName!='/' && *ProgName!='\\';);
 	++ProgName;
 #if defined(MSDOS)||defined(_WIN32)
 	if((dot=strchr(ProgName, '.'))) *dot=0;
@@ -212,7 +215,7 @@ GetParams(int	*n,
 		defCmd += strspn(defCmd, "\n\r");
 	}
 	for(;;) {
-		CmdFile=0;
+		char *CmdFile = NULL;
 		if(argc && argv[0][0]=='-' && argv[0][1]=='=') {
 			CmdFile = argv[0]+2;
 			++argv;
@@ -231,8 +234,9 @@ GetParams(int	*n,
 			defCmd += strspn(defCmd, "\n\r");
 		}
 		
-		IsPipe = !strncmp(CmdFile, "@@", 2);
-		fp = IsPipe
+		int	IsPipe = !strncmp(CmdFile, "@@", 2);
+		
+		FILE	*fp = IsPipe
 		? popen(CmdFile+2, "r")
 		: strcmp(CmdFile, "-")
 		? fopen(CmdFile, "r")
@@ -241,9 +245,8 @@ GetParams(int	*n,
 		
 		if(!fp) {
 			if(defCmd) continue;
-			fprintf(stderr, "Unable to open command file %s\n",
-							CmdFile);
-			exit(1);
+			fprintf(stderr, "Unable to open command file %s\n", CmdFile);
+			exit(IRSTLM_CMD_ERROR_IO);
 		}
 		while(GetLine(fp, LINSIZ, Line) && strcmp(Line, "\\End")) {
 			StoreCmdLine(Line);
@@ -256,11 +259,10 @@ GetParams(int	*n,
 			else
 				fclose(fp);
 		}
-		CmdFile = NULL;
 	}
 	if(DefCmd) free(DefCmd);
 	
-//	while(argc && **argv=='-'){
+	//	while(argc && **argv=='-'){
 	while(argc){
 		if (**argv=='-'){
 			s=strchr(*argv, '=');
@@ -356,14 +358,6 @@ FullPrintParams(int	TypeFlag,
 {
 	return PrintParams4(TypeFlag, ValFlag, MsgFlag, fp);
 }
-/*
- int 
- PrintParams(int		ValFlag,
- FILE	*fp, int MsgFlag)
- {
- return PrintParams4(MsgFlag, ValFlag, 0, fp);
- }
- */
 
 static int 
 PrintParams4(int	TypeFlag,
@@ -433,7 +427,7 @@ BuildCmdList(Cmd_T	**cmdp,
 	if(!*cmdSz) {
 		if(!(cmds=*cmdp=malloc((1+(*cmdSz=BUFSIZ))*sizeof(Cmd_T)))) {
 			fprintf(stderr, "BuildCmdList(): malloc() failed\n");
-			exit(-1);
+			exit(IRSTLM_CMD_ERROR_MEMORY);
 		}
 	} else {
 		for(cmds=*cmdp; cmds[cmdN].Name; ++cmdN);
@@ -445,7 +439,7 @@ BuildCmdList(Cmd_T	**cmdp,
 			if(!cmds) {
 				fprintf(stderr,
 								"BuildCmdList(): realloc() failed\n");
-				exit(-1);
+				exit(IRSTLM_CMD_ERROR_MEMORY);
 			}
 		}
 		for(j=0; j<cmdN&&strcmp(cmds[j].Name, ParName)<0; j++);
@@ -456,6 +450,8 @@ BuildCmdList(Cmd_T	**cmdp,
 		cmd->Val = va_arg(args, void*);
 		cmd->Msg = 0;
 		cmd->Flag = 0;
+		cmd->p = 0;
+		
 		switch(cmd->Type&~CMDMSG) {
 			case CMDENUMTYPE:	/* get the pointer to Enum_T struct */
 			case CMDFLAGTYPE:
@@ -483,10 +479,11 @@ BuildCmdList(Cmd_T	**cmdp,
 				*((int**)cmd->p)[1] = 0;
 				break;
 			case CMDBOOLTYPE:
-				cmd->Type = CMDENUMTYPE|(cmd->Type&CMDMSG);
 				cmd->p = BoolEnum;
-				cmd->Flag = 1;
 				break;
+				//cmd->p = (Bool_T*)calloc(1, sizeof(Bool_T));
+				//				cmd->p = va_arg(args, void*);
+//				cmd->p = BoolEnum;
 			case CMDDOUBLETYPE:	/* nothing else is needed	 */
 			case CMDFLOATTYPE:
 			case CMDINTTYPE:
@@ -496,7 +493,7 @@ BuildCmdList(Cmd_T	**cmdp,
 				fprintf(stderr, "%s: %s %d %s \"%s\"\n",
 								"BuildCmdList()", "Unknown Type",
 								cmd->Type&~CMDMSG, "for parameter", cmd->Name);
-				exit(1);
+				exit(IRSTLM_CMD_ERROR_DATA);
 		}
 		if(cmd->Type&CMDMSG) {
 			cmd->Type&=~CMDMSG;
@@ -515,7 +512,7 @@ CmdError(char	*opt)
 	fprintf(stderr, "Invalid option \"%s\"\n", opt);
 	fprintf(stderr, "This program expects the following parameters:\n");
 	PrintParams4(TRUE, FALSE, TRUE, stderr);
-	exit(0);
+	exit(IRSTLM_CMD_ERROR_DATA);
 	return 0;
 }
 
@@ -523,6 +520,7 @@ static int
 FreeParam(Cmd_T	*cmd)
 {
 	switch(cmd->Type) {
+		case CMDBOOLTYPE2:
 		case CMDSUBRANGETYPE:
 		case CMDGTETYPE:
 		case CMDLTETYPE:
@@ -560,6 +558,10 @@ PrintParam(Cmd_T	*cmd,
 			if(TypeFlag) fprintf(fp, " [float]");
 			if(ValFlag) fprintf(fp, ": %22.15e", *(float *)cmd->Val);
 			break;
+		case CMDBOOLTYPE2:
+		case CMDBOOLTYPE:
+			PrintBool(cmd, TypeFlag, ValFlag, fp);
+			break;
 		case CMDENUMTYPE:
 			PrintEnum(cmd, TypeFlag, ValFlag, fp);
 			break;
@@ -609,7 +611,7 @@ PrintParam(Cmd_T	*cmd,
 							cmd->Type,
 							"for parameter",
 							cmd->Name);
-			exit(1);
+			exit(IRSTLM_CMD_ERROR_DATA);
 	}
 	fprintf(fp, ":");
 	//	fprintf(fp, "\n");
@@ -654,8 +656,8 @@ GetLine(FILE	*fp,
 
 static int 
 Scan(char	*ProgName,
-     Cmd_T	*cmds,
-     char	*Line)
+		 Cmd_T	*cmds,
+		 char	*Line)
 {
 	char	*q,
 	*p;
@@ -664,7 +666,7 @@ Scan(char	*ProgName,
 	HasToMatch = FALSE,
 	c0,
 	c;
-
+	
 	p = Line+strspn(Line, SepString);
 	if(!(hl=strcspn(p, SepString))) return 0;
 	if(ProgName&&(q=strchr(p, '/')) && q-p<hl) {
@@ -713,7 +715,7 @@ SetParam(Cmd_T	*cmd,
 				fprintf(stderr,
 								"Float value required for parameter \"%s\"\n",
 								cmd->Name);
-				exit(1);
+				exit(IRSTLM_CMD_ERROR_DATA);
 			}
 			break;
 		case CMDFLOATTYPE:
@@ -721,9 +723,13 @@ SetParam(Cmd_T	*cmd,
 				fprintf(stderr,
 								"Float value required for parameter \"%s\"\n",
 								cmd->Name);
-				exit(1);
+				exit(IRSTLM_CMD_ERROR_DATA);
 			}
 			break;
+		case CMDBOOLTYPE2:
+		case CMDBOOLTYPE:
+			SetBool(cmd, s);
+			break;
 		case CMDENUMTYPE:
 			SetEnum(cmd, s);
 			break;
@@ -731,11 +737,14 @@ SetParam(Cmd_T	*cmd,
 			SetFlag(cmd, s);
 			break;
 		case CMDINTTYPE:
-			if(sscanf(s, "%i", (int*)cmd->Val)!=1) {
+			/*They are the same when used for output, e.g. with printf, but different when used as input specifier e.g. with scanf, where %d scans an integer as a signed decimal number, but %i defaults to decimal but also allows hexadecimal (if preceded by "0x") and octal if preceded by "0".
+			 So "033" would be 27 with %i but 33 with %d.
+			 */
+			if(sscanf(s, "%d", (int*)cmd->Val)!=1) {
 				fprintf(stderr,
 								"Integer value required for parameter \"%s\"\n",
 								cmd->Name);
-				exit(1);
+				exit(IRSTLM_CMD_ERROR_DATA);
 			}
 			break;
 		case CMDSTRINGTYPE:
@@ -766,7 +775,7 @@ SetParam(Cmd_T	*cmd,
 							cmd->Type,
 							"for parameter",
 							cmd->Name);
-			exit(1);
+			exit(IRSTLM_CMD_ERROR_DATA);
 	}
 	cmd->ArgStr = strdup(s);
 	
@@ -777,6 +786,22 @@ SetParam(Cmd_T	*cmd,
 	return 0;
 }
 
+static int 
+SetBool(Cmd_T	*cmd,
+				char	*s)
+{
+	Bool_T	*en;
+	
+	for(en=(Bool_T*)cmd->p; en->Name; en++) {
+		if(*en->Name && !strcmp(s, en->Name)) {
+			*(char*)cmd->Val = en->Idx;
+			return 0;
+		}
+	}
+	return BoolError(cmd, s);
+}
+
+
 static int 
 SetEnum(Cmd_T	*cmd,
 				char	*s)
@@ -802,6 +827,16 @@ EnumIdx(Enum_T	*en,
 	return -1;
 }
 
+char
+BoolIdx(Bool_T	*en,
+				char	*s)
+{
+	if(en) for(; en->Name; en++) {
+		if(*en->Name && !strcmp(s, en->Name)) return en->Idx;
+	}
+	return -1;
+}
+
 char *
 EnumStr(Enum_T	*en,
 				int	i)
@@ -810,6 +845,14 @@ EnumStr(Enum_T	*en,
 	return 0;
 }
 
+char *
+BoolStr(Bool_T	*en,
+				int	i)
+{
+	if(en) for(; en->Name; en++) if(en->Idx==i) return en->Name;
+	return 0;
+}
+
 static int 
 SetFlag(Cmd_T	*cmd,
 				char	*s)
@@ -819,8 +862,8 @@ SetFlag(Cmd_T	*cmd,
 	
 	for(; (l=strcspn(s, "+"))>0; s+=l,s+=!!*s) {
 		for(en=(Enum_T*)cmd->p;
-		    en->Name&&(l!=strlen(en->Name)||strncmp(s, en->Name, l));
-		    en++);
+				en->Name&&(l!=strlen(en->Name)||strncmp(s, en->Name, l));
+				en++);
 		if(!en->Name) return EnumError(cmd, s);
 		*(int*)cmd->Val |= en->Idx;
 	}
@@ -832,12 +875,15 @@ SetSubrange(Cmd_T	*cmd,
 						char	*s)
 {
 	int	n;
-	
-	if(sscanf(s, "%i", &n)!=1) {
+
+	/*They are the same when used for output, e.g. with printf, but different when used as input specifier e.g. with scanf, where %d scans an integer as a signed decimal number, but %i defaults to decimal but also allows hexadecimal (if preceded by "0x") and octal if preceded by "0".
+	 So "033" would be 27 with %i but 33 with %d.
+	 */
+	if(sscanf(s, "%d", &n)!=1) {
 		fprintf(stderr,
 						"Integer value required for parameter \"%s\"\n",
 						cmd->Name);
-		exit(1);
+		exit(IRSTLM_CMD_ERROR_DATA);
 	}
 	if(n < *(int*)cmd->p || n > *((int*)cmd->p+1)) {
 		return SubrangeError(cmd, n);
@@ -848,15 +894,18 @@ SetSubrange(Cmd_T	*cmd,
 
 static int 
 SetGte(Cmd_T	*cmd,
-       char	*s)
+			 char	*s)
 {
 	int	n;
 	
-	if(sscanf(s, "%i", &n)!=1) {
+	/*They are the same when used for output, e.g. with printf, but different when used as input specifier e.g. with scanf, where %d scans an integer as a signed decimal number, but %i defaults to decimal but also allows hexadecimal (if preceded by "0x") and octal if preceded by "0".
+		So "033" would be 27 with %i but 33 with %d.
+		*/
+	if(sscanf(s, "%d", &n)!=1) {
 		fprintf(stderr,
 						"Integer value required for parameter \"%s\"\n",
 						cmd->Name);
-		exit(1);
+		exit(IRSTLM_CMD_ERROR_DATA);
 	}
 	if(n<*(int*)cmd->p) {
 		return GteError(cmd, n);
@@ -884,15 +933,18 @@ SetNumArray(Cmd_T	*cmd,
 
 static int 
 SetLte(Cmd_T	*cmd,
-       char	*s)
+			 char	*s)
 {
 	int	n;
 	
-	if(sscanf(s, "%i", &n)!=1) {
+	/*They are the same when used for output, e.g. with printf, but different when used as input specifier e.g. with scanf, where %d scans an integer as a signed decimal number, but %i defaults to decimal but also allows hexadecimal (if preceded by "0x") and octal if preceded by "0".
+	 So "033" would be 27 with %i but 33 with %d.
+	 */
+	if(sscanf(s, "%d", &n)!=1) {
 		fprintf(stderr,
 						"Integer value required for parameter \"%s\"\n",
 						cmd->Name);
-		exit(1);
+		exit(IRSTLM_CMD_ERROR_DATA);
 	}
 	if(n > *(int*)cmd->p) {
 		return LteError(cmd, n);
@@ -914,7 +966,24 @@ EnumError(Cmd_T	*cmd,
 		if(*en->Name) fprintf(stderr, " %s\n", en->Name);
 	}
 	fprintf(stderr, "\n");
-	exit(1);
+	exit(IRSTLM_CMD_ERROR_DATA);
+	return 0;
+}
+
+static int 
+BoolError(Cmd_T	*cmd,
+					char	*s)
+{
+	Bool_T	*en;
+	
+	fprintf(stderr,
+					"Invalid value \"%s\" for parameter \"%s\"\n", s, cmd->Name);
+	fprintf(stderr, "Valid values are:\n");
+	for(en=(Bool_T*)cmd->p; en->Name; en++) {
+		if(*en->Name) fprintf(stderr, " %s\n", en->Name);
+	}
+	fprintf(stderr, "\n");
+	exit(IRSTLM_CMD_ERROR_DATA);
 	return 0;
 }
 
@@ -926,7 +995,7 @@ GteError(Cmd_T	*cmd,
 					"Value %d out of range for parameter \"%s\"\n", n, cmd->Name);
 	fprintf(stderr, "Valid values must be greater than or equal to %d\n",
 					*(int*)cmd->p);
-	exit(1);
+	exit(IRSTLM_CMD_ERROR_DATA);
 	return 0;
 }
 
@@ -938,7 +1007,7 @@ LteError(Cmd_T	*cmd,
 					"Value %d out of range for parameter \"%s\"\n", n, cmd->Name);
 	fprintf(stderr, "Valid values must be less than or equal to %d\n",
 					*(int*)cmd->p);
-	exit(1);
+	exit(IRSTLM_CMD_ERROR_DATA);
 	return 0;
 }
 
@@ -950,7 +1019,7 @@ SubrangeError(Cmd_T	*cmd,
 					"Value %d out of range for parameter \"%s\"\n", n, cmd->Name);
 	fprintf(stderr, "Valid values range from %d to %d\n",
 					*(int*)cmd->p, *((int*)cmd->p+1));
-	exit(1);
+	exit(IRSTLM_CMD_ERROR_DATA);
 	return 0;
 }
 
@@ -961,11 +1030,13 @@ PrintEnum(Cmd_T	*cmd,
 					FILE	*fp)
 {
 	Enum_T	*en;
-	char	*sep="";
 	
 	fprintf(fp, "%s", cmd->Name);
 	if(TypeFlag) {
 		fprintf(fp, " [enum { ");
+		
+		char	*sep="";
+		
 		for(en=(Enum_T*)cmd->p; en->Name; en++) {
 			if(*en->Name) {
 				fprintf(fp, "%s%s", sep, en->Name);
@@ -985,6 +1056,39 @@ PrintEnum(Cmd_T	*cmd,
 	return 0;
 }
 
+static int 
+PrintBool(Cmd_T	*cmd,
+					int	TypeFlag,
+					int	ValFlag,
+					FILE	*fp)
+{
+	Bool_T	*en;
+	
+	fprintf(fp, "%s", cmd->Name);
+	if(TypeFlag) {
+		fprintf(fp, " [enum { ");
+		
+		char	*sep="";
+		
+		for(en=(Bool_T*)cmd->p; en->Name; en++) {
+			if(*en->Name) {
+				fprintf(fp, "%s%s", sep, en->Name);
+				sep=", ";
+			}
+		}
+		fprintf(fp, " }]");
+	}
+	if(ValFlag) {
+		for(en=(Bool_T*)cmd->p; en->Name; en++) {
+			if(*en->Name && en->Idx==*(int*)cmd->Val) {
+				fprintf(fp, ": %s", en->Name);
+			}
+		}
+	}
+	//	fprintf(fp, "\n");
+	return 0;
+}
+
 static int 
 PrintFlag(Cmd_T	*cmd,
 					int	TypeFlag,
@@ -1115,10 +1219,8 @@ static char **
 str2array(char	*s,
 					char	*sep)
 {
-	char	*p,
-	**a;
-	int	n = 0,
-	l;
+	char	*p, **a;
+	int	n = 0;
 	
 	if(!sep) sep = SepString;
 	p = s += strspn(s, sep);
@@ -1132,7 +1234,7 @@ str2array(char	*s,
 	p = s;
 	n = 0;
 	while(*p) {
-		l = strcspn(p, sep);
+		int l = strcspn(p, sep);
 		a[n] = malloc(l+1);
 		memcpy(a[n], p, l);
 		a[n][l] = 0;
@@ -1196,9 +1298,10 @@ StoreCmdLine(char	*s)
 		if(!CmdLines) {
 			fprintf(stderr, "%s\n",
 							"StoreCmdLine(): malloc() failed");
-			exit(-1);
+			exit(IRSTLM_CMD_ERROR_MEMORY);
 		}
 	}
 	CmdLines[CmdLinesL++] = strdup(s);
 	return 0;
 }
+
