@@ -20,7 +20,6 @@
 
  ******************************************************************************/
 
-using namespace std;
 
 #include <iostream>
 #include <fstream>
@@ -32,6 +31,9 @@ using namespace std;
 #include "math.h"
 #include "lmContainer.h"
 
+using namespace std;
+using namespace irstlm;
+
 /********************************/
 void print_help(int TypeFlag=0){
   std::cerr << std::endl << "compile-lm - compiles an ARPA format LM into an IRSTLM format one" << std::endl;
@@ -54,7 +56,6 @@ void usage(const char *msg = 0)
 	if (!msg){
 		print_help();
 	}
-	exit(1);
 }
 
 int main(int argc, char **argv)
@@ -67,12 +68,13 @@ int main(int argc, char **argv)
 	bool sent_PP_flag = false;
 	bool invert = false;
 	bool sscore = false;
+	bool ngramscore = false;
 	bool skeepunigrams = false;
 	
 	int debug = 0;
   bool memmap = false;
-  int requiredMaxlev = 1000;
-  int dub = 10000000;
+  int requiredMaxlev = IRSTLM_REQUIREDMAXLEV_DEFAULT;
+  int dub = IRSTLM_DUB_DEFAULT;
   int randcalls = 0;
   float ngramcache_load_factor = 0.0;
   float dictionary_load_factor = 0.0;
@@ -93,7 +95,9 @@ int main(int argc, char **argv)
 								"r", CMDINTTYPE|CMDMSG, &randcalls, "computes N random calls on the specified text file",
                 "score", CMDBOOLTYPE|CMDMSG, &sscore, "computes log-prob scores of n-grams from standard input",
 								"s", CMDBOOLTYPE|CMDMSG, &sscore, "computes log-prob scores of n-grams from standard input",
-                "debug", CMDINTTYPE|CMDMSG, &debug, "verbose output for --eval option; default is 0",
+                "ngramscore", CMDBOOLTYPE|CMDMSG, &ngramscore, "computes log-prob scores of the last n-gram  before an _END_NGRAM_ symbol from standard input",
+                "ns", CMDBOOLTYPE|CMDMSG, &ngramscore, "computes log-prob scores of the last n-gram  before an _END_NGRAM_ symbol from standard input",
+								"debug", CMDINTTYPE|CMDMSG, &debug, "verbose output for --eval option; default is 0",
 								"d", CMDINTTYPE|CMDMSG, &debug, "verbose output for --eval option; default is 0",
                 "level", CMDINTTYPE|CMDMSG, &requiredMaxlev, "maximum level to load from the LM; if value is larger than the actual LM order, the latter is taken",
 								"l", CMDINTTYPE|CMDMSG, &requiredMaxlev, "maximum level to load from the LM; if value is larger than the actual LM order, the latter is taken",
@@ -115,6 +119,7 @@ int main(int argc, char **argv)
 	
 	if (argc == 1){
 		usage();
+		exit_error(IRSTLM_NO_ERROR);
 	}
 	
 	for(int i=1; i < argc; i++) {
@@ -128,25 +133,19 @@ int main(int argc, char **argv)
 	
 	if (help){
 		usage();
+		exit_error(IRSTLM_NO_ERROR);
 	}	
 
   if (files.size() > 2) {
-    usage("Warning: Too many arguments");
+    usage();
+		exit_error(IRSTLM_ERROR_DATA,"Warning: Too many arguments");
   }
 
   if (files.size() < 1) {
-    usage("Warning: Please specify a LM file to read from");
+    usage();
+		exit_error(IRSTLM_ERROR_DATA,"Warning: Please specify a LM file to read from");
   }
 
-  //Define output type of table
-	OUTFILE_TYPE outtype;
-  if (textoutput)
-    outtype=TEXT;
-  else if (seval != NULL || sscore)
-    outtype=NONE;
-  else
-    outtype=BINARY;
-
   std::string infile = files[0];
   std::string outfile = "";
 
@@ -165,12 +164,13 @@ int main(int argc, char **argv)
     outfile+=(textoutput?".lm":".blm");
   } else{
     outfile = files[1];
-	}
+  }
 	
   std::cerr << "inpfile: " << infile << std::endl;
 	std::cerr << "outfile: " << outfile << std::endl;
   if (seval!=NULL) std::cerr << "evalfile: " << seval << std::endl;
   if (sscore==true) std::cerr << "interactive: " << sscore << std::endl;
+  if (ngramscore==true) std::cerr << "interactive for ngrams only: " << ngramscore << std::endl;
   if (memmap) std::cerr << "memory mapping: " << memmap << std::endl;
   std::cerr << "loading up to the LM level " << requiredMaxlev << " (if any)" << std::endl;
   std::cerr << "dub: " << dub<< std::endl;
@@ -182,10 +182,8 @@ int main(int argc, char **argv)
 
 
   //checking the language model type
-  lmContainer* lmt=NULL;
-
-  lmt = lmt->CreateLanguageModel(infile,ngramcache_load_factor,dictionary_load_factor);
-
+  lmContainer* lmt = lmContainer::CreateLanguageModel(infile,ngramcache_load_factor,dictionary_load_factor);
+	
   //let know that table has inverted n-grams
   if (invert) lmt->is_inverted(invert);
 
@@ -193,6 +191,8 @@ int main(int argc, char **argv)
 
   lmt->load(infile);
 
+	lmt->print_table_stat();
+	
   //CHECK this part for sfilter to make it possible only for LMTABLE
   if (sfilter != NULL) {
     lmContainer* filtered_lmt = NULL;
@@ -249,6 +249,7 @@ int main(int argc, char **argv)
         if (debug==1) {
           std::cout << ng.dict->decode(*ng.wordp(1)) << " [" << lmt->maxlevel()-bol << "]" << " ";
           std::cout << std::endl;
+          std::cout.flush();
         }
 
         if ((n % 100000)==0) {
@@ -268,6 +269,14 @@ int main(int argc, char **argv)
         debug = (debug>4)?4:debug;
         std::cerr << "Maximum debug value for this LM type: " << debug << std::endl;
       }
+      if (lmt->getLanguageModelType() == _IRSTLM_LMMACRO) {
+        debug = (debug>4)?4:debug;
+        std::cerr << "Maximum debug value for this LM type: " << debug << std::endl;
+      }
+      if (lmt->getLanguageModelType() == _IRSTLM_LMCLASS) {
+        debug = (debug>4)?4:debug;
+        std::cerr << "Maximum debug value for this LM type: " << debug << std::endl;
+      }
       std::cerr << "Start Eval" << std::endl;
       std::cerr << "OOV code: " << lmt->getDict()->oovcode() << std::endl;
       ngram ng(lmt->getDict());
@@ -276,7 +285,7 @@ int main(int argc, char **argv)
 
       //			if (debug>0) std::cout.precision(8);
       std::fstream inptxt(seval,std::ios::in);
-
+			
       int Nbo=0, Nw=0,Noov=0;
       double logPr=0,PP=0,PPwp=0,Pr;
 
@@ -284,31 +293,33 @@ int main(int argc, char **argv)
       int sent_Nbo=0, sent_Nw=0,sent_Noov=0;
       double sent_logPr=0,sent_PP=0,sent_PPwp=0;
 
-
-      ng.dict->incflag(1);
-      int bos=ng.dict->encode(ng.dict->BoS());
-      int eos=ng.dict->encode(ng.dict->EoS());
-      ng.dict->incflag(0);
-
+      int bos=lmt->addWord(lmt->getDict()->BoS());
+      int eos=lmt->addWord(lmt->getDict()->EoS());
+			
       double bow;
       int bol=0;
+      ngram_state_t msidx;
       char *msp;
       unsigned int statesize;
 
       lmt->dictionary_incflag(1);
 
       while(inptxt >> ng) {
+				VERBOSE(3,"read ng:|" << ng << "|" << std::endl);
 
         if (ng.size>lmt->maxlevel()) ng.size=lmt->maxlevel();
 
         // reset ngram at begin of sentence
         if (*ng.wordp(1)==bos) {
-          ng.size=1;
+					ng.size=1;
           continue;
         }
 
         if (ng.size>=1) {
-          Pr=lmt->clprob(ng,&bow,&bol,&msp,&statesize);
+					VERBOSE(3,"computing clprob ng:|" << ng << "|" << std::endl);
+//          Pr=lmt->clprob(ng,&bow,&bol,&msp,&statesize);
+          Pr=lmt->clprob(ng,&bow,&bol,&msidx,&msp,&statesize);
+					
           logPr+=Pr;
           sent_logPr+=Pr;
 
@@ -316,20 +327,23 @@ int main(int argc, char **argv)
             std::cout << ng.dict->decode(*ng.wordp(1)) << " [" << ng.size-bol << "]" << " ";
             if (*ng.wordp(1)==eos) std::cout << std::endl;
           }
-          if (debug==2) {
+          else if (debug==2) {
             std::cout << ng << " [" << ng.size-bol << "-gram]" << " " << Pr;
             std::cout << std::endl;
+            std::cout.flush();
           }
-          if (debug==3) {
+          else if (debug==3) {
             std::cout << ng << " [" << ng.size-bol << "-gram]" << " " << Pr << " bow:" << bow;
             std::cout << std::endl;
+            std::cout.flush();
           }
-          if (debug==4) {
-            std::cout << ng << " [" << ng.size-bol << "-gram: recombine:" << statesize << " state:" << (void*) msp << "] [" << ng.size+1-((bol==0)?(1):bol) << "-gram: bol:" << bol << "] " << Pr << " bow:" << bow;
+          else if (debug==4) {
+            std::cout << ng << " [" << ng.size-bol << "-gram: recombine:" << statesize << " ngramstate:" << msidx << " state:" << (void*) msp << "] [" << ng.size+1-((bol==0)?(1):bol) << "-gram: bol:" << bol << "] " << Pr << " bow:" << bow;
             std::cout << std::endl;
+            std::cout.flush();
           }
-          if (debug>4) {
-            std::cout << ng << " [" << ng.size-bol << "-gram: recombine:" << statesize << " state:" << (void*) msp << "] [" << ng.size+1-((bol==0)?(1):bol) << "-gram: bol:" << bol << "] " << Pr << " bow:" << bow;
+          else if (debug>4) {
+            std::cout << ng << " [" << ng.size-bol << "-gram: recombine:" << statesize << " ngramstate:" << msidx << " state:" << (void*) msp << "] [" << ng.size+1-((bol==0)?(1):bol) << "-gram: bol:" << bol << "] " << Pr << " bow:" << bow;
             double totp=0.0;
             int oldw=*ng.wordp(1);
             double oovp=lmt->getlogOOVpenalty();
@@ -343,6 +357,7 @@ int main(int argc, char **argv)
             if ( totp < (1.0 - 1e-5) || totp > (1.0 + 1e-5))
               std::cout << "  [t=" << totp << "] POSSIBLE ERROR";
             std::cout << std::endl;
+            std::cout.flush();
 
             lmt->setlogOOVpenalty((double)oovp);
           }
@@ -368,6 +383,7 @@ int main(int argc, char **argv)
                       << " sent_Nbo=" << sent_Nbo
                       << " sent_Noov=" << sent_Noov
                       << " sent_OOV=" << (float)sent_Noov/sent_Nw * 100.0 << "%" << std::endl;
+            std::cout.flush();
             //reset statistics for sentence based Perplexity
             sent_Nw=sent_Noov=sent_Nbo=0;
             sent_logPr=0.0;
@@ -377,8 +393,10 @@ int main(int argc, char **argv)
             std::cerr << ".";
             lmt->check_caches_levels();
           }
-
+					
+					VERBOSE(3,"computing clprob END" << std::endl);
         }
+				VERBOSE(3,"read END" << std::endl);
       }
 
       PP=exp((-logPr * log(10.0)) /Nw);
@@ -393,9 +411,9 @@ int main(int argc, char **argv)
                 << " OOV=" << (float)Noov/Nw * 100.0 << "%";
       if (debug) std::cout << " logPr=" <<  logPr;
       std::cout << std::endl;
+      std::cout.flush();
 
-      lmt->used_caches();
-      lmt->stat();
+      if (debug>1) lmt->used_caches();
 
       if (debug>1) lmt->stat();
 
@@ -414,6 +432,8 @@ int main(int argc, char **argv)
     unsigned int n=0;
 
     std::cout.setf(ios::scientific);
+    std::cout.setf(ios::fixed);
+    std::cout.precision(2);
     std::cout << "> ";
 
     lmt->dictionary_incflag(1);
@@ -442,7 +462,59 @@ int main(int argc, char **argv)
       std::cout << "> ";
     }
     std::cout << std::endl;
+    std::cout.flush();
+    if (debug>1) lmt->used_caches();
+
     if (debug>1) lmt->stat();
+
+    delete lmt;
+    return 0;
+  }
+	
+	
+  if (ngramscore == true) {
+		
+		const char* _END_NGRAM_="_END_NGRAM_";
+    ngram ng(lmt->getDict());
+		
+		double Pr;
+		double bow;
+		int bol=0;
+		ngram_state_t msidx;
+		char *msp;
+		unsigned int statesize;
+
+		std::cout.setf(ios::fixed);
+		std::cout.precision(2);
+		
+		ng.dict->incflag(1);
+		int endngram=ng.dict->encode(_END_NGRAM_);
+		ng.dict->incflag(0);
+		
+    while(std::cin >> ng) {
+      // compute score for the last ngram when endngram symbols is found
+      // and reset ngram
+      if (*ng.wordp(1)==endngram) {
+				ng.shift();
+				if (ng.size>=lmt->maxlevel()) {
+					ng.size=lmt->maxlevel();
+				}
+				
+//				Pr=lmt->clprob(ng,&bow,&bol,&msp,&statesize);
+				Pr=lmt->clprob(ng,&bow,&bol,&msidx, &msp,&statesize);
+#ifndef OUTPUT_SUPPRESSED
+				std::cout << ng << " [" << ng.size-bol << "-gram: recombine:" << statesize << " ngramstate:" << msidx << " state:" << (void*) msp << "] [" << ng.size+1-((bol==0)?(1):bol) << "-gram: bol:" << bol << "] " << Pr << " bow:" << bow;
+				std::cout << std::endl;
+    				std::cout.flush();
+#endif
+        ng.size=0;
+      }
+    }
+
+    if (debug>1) lmt->used_caches();
+
+    if (debug>1) lmt->stat();
+
     delete lmt;
     return 0;
   }
