@@ -18,8 +18,6 @@
 
  ******************************************************************************/
 
-using namespace std;
-
 #include <iostream>
 #include <fstream>
 #include <sstream>
@@ -31,8 +29,11 @@ using namespace std;
 #include "util.h"
 #include "math.h"
 #include "lmContainer.h"
-/********************************/
 
+#define MAX_N   100
+/********************************/
+using namespace std;
+using namespace irstlm;
 
 inline void error(const char* message)
 {
@@ -68,7 +69,6 @@ void usage(const char *msg = 0)
   else{
 		print_help();
 	}
-	exit(1);
 }
 
 int main(int argc, char **argv)
@@ -82,8 +82,8 @@ int main(int argc, char **argv)
 	int order = 0;
 	int debug = 0;
   int memmap = 0;
-  int requiredMaxlev = 1000;
-  int dub = 10000000;
+  int requiredMaxlev = IRSTLM_REQUIREDMAXLEV_DEFAULT;
+  int dub = IRSTLM_DUB_DEFAULT;
   float ngramcache_load_factor = 0.0;
   float dictionary_load_factor = 0.0;
 	
@@ -91,39 +91,38 @@ int main(int argc, char **argv)
   std::vector<std::string> files;
 	
 	DeclareParams((char*)
-								
-								"learn", CMDSTRINGTYPE|CMDMSG, &slearn, "learn optimal interpolation for text-file; default is false",
-								"l", CMDSTRINGTYPE|CMDMSG, &slearn, "learn optimal interpolation for text-file; default is false",
-								
-								"order", CMDINTTYPE|CMDMSG, &order, "order of n-grams used in --learn (optional)",
-								"o", CMDINTTYPE|CMDMSG, &order, "order of n-grams used in --learn (optional)",
-								
+					
+		"learn", CMDSTRINGTYPE|CMDMSG, &slearn, "learn optimal interpolation for text-file; default is false",
+		"l", CMDSTRINGTYPE|CMDMSG, &slearn, "learn optimal interpolation for text-file; default is false",
+		"order", CMDINTTYPE|CMDMSG, &order, "order of n-grams used in --learn (optional)",
+		"o", CMDINTTYPE|CMDMSG, &order, "order of n-grams used in --learn (optional)",						
                 "eval", CMDSTRINGTYPE|CMDMSG, &seval, "computes perplexity of the specified text file",
-								"e", CMDSTRINGTYPE|CMDMSG, &seval, "computes perplexity of the specified text file",
+		"e", CMDSTRINGTYPE|CMDMSG, &seval, "computes perplexity of the specified text file",
 								
+                "DictionaryUpperBound", CMDINTTYPE|CMDMSG, &dub, "dictionary upperbound to compute OOV word penalty: default 10^7",
                 "dub", CMDINTTYPE|CMDMSG, &dub, "dictionary upperbound to compute OOV word penalty: default 10^7",
-
                 "score", CMDBOOLTYPE|CMDMSG, &score, "computes log-prob scores of n-grams from standard input",
-								"s", CMDBOOLTYPE|CMDMSG, &score, "computes log-prob scores of n-grams from standard input",
+		"s", CMDBOOLTYPE|CMDMSG, &score, "computes log-prob scores of n-grams from standard input",
 								
                 "debug", CMDINTTYPE|CMDMSG, &debug, "verbose output for --eval option; default is 0",
-								"d", CMDINTTYPE|CMDMSG, &debug, "verbose output for --eval option; default is 0",
+		"d", CMDINTTYPE|CMDMSG, &debug, "verbose output for --eval option; default is 0",
                 "memmap", CMDINTTYPE|CMDMSG, &memmap, "uses memory map to read a binary LM",
-								"mm", CMDINTTYPE|CMDMSG, &memmap, "uses memory map to read a binary LM",
-								"sentence", CMDBOOLTYPE|CMDMSG, &sent_PP_flag, "computes perplexity at sentence level (identified through the end symbol)",
+		"mm", CMDINTTYPE|CMDMSG, &memmap, "uses memory map to read a binary LM",
+		"sentence", CMDBOOLTYPE|CMDMSG, &sent_PP_flag, "computes perplexity at sentence level (identified through the end symbol)",
                 "dict_load_factor", CMDFLOATTYPE|CMDMSG, &dictionary_load_factor, "sets the load factor for ngram cache; it should be a positive real value; default is 0",
                 "ngram_load_factor", CMDFLOATTYPE|CMDMSG, &ngramcache_load_factor, "sets the load factor for ngram cache; it should be a positive real value; default is false",
                 "level", CMDINTTYPE|CMDMSG, &requiredMaxlev, "maximum level to load from the LM; if value is larger than the actual LM order, the latter is taken",
-								"lev", CMDINTTYPE|CMDMSG, &requiredMaxlev, "maximum level to load from the LM; if value is larger than the actual LM order, the latter is taken",
+		"lev", CMDINTTYPE|CMDMSG, &requiredMaxlev, "maximum level to load from the LM; if value is larger than the actual LM order, the latter is taken",
 								
-								"Help", CMDBOOLTYPE|CMDMSG, &help, "print this help",
-								"h", CMDBOOLTYPE|CMDMSG, &help, "print this help",
+		"Help", CMDBOOLTYPE|CMDMSG, &help, "print this help",
+		"h", CMDBOOLTYPE|CMDMSG, &help, "print this help",
 								
 								(char *)NULL
 								);
 	
 	if (argc == 1){
 		usage();
+		exit_error(IRSTLM_NO_ERROR);
 	}
 	
 	for(int i=1; i < argc; i++) {
@@ -134,14 +133,17 @@ int main(int argc, char **argv)
 	
 	if (help){
 		usage();
+		exit_error(IRSTLM_NO_ERROR);
 	}
 
   if (files.size() > 2) {
-    usage("Warning: Too many arguments");
+    usage();
+		exit_error(IRSTLM_ERROR_DATA,"Too many arguments");
   }
 	
   if (files.size() < 1) {
-    usage("Warning: specify a LM list file to read from");
+    usage();
+		exit_error(IRSTLM_ERROR_DATA,"Must pecify a LM list file to read from");
   }
 
   std::string infile = files[0];
@@ -169,10 +171,10 @@ int main(int argc, char **argv)
 
   std::cerr << "dub: " << dub<< std::endl;
 
-  lmContainer *lmt[100], *start_lmt[100]; //interpolated language models
-  std::string lmf[100]; //lm filenames
+  lmContainer *lmt[MAX_N], *start_lmt[MAX_N]; //interpolated language models
+  std::string lmf[MAX_N]; //lm filenames
 
-  float w[100]; //interpolation weights
+  float w[MAX_N]; //interpolation weights
   int N;
 
 
@@ -193,17 +195,16 @@ int main(int argc, char **argv)
 
   N=atoi(words[1]);
   std::cerr << "Number of LMs: " << N << "..." << std::endl;
-  if(N > 100) {
-    std::cerr << "Can't interpolate more than 100 language models." << std::endl;
-    exit(1);
+  if(N > MAX_N) {
+		exit_error(IRSTLM_ERROR_DATA,"Can't interpolate more than MAX_N language models");
+		
   }
 
   for (int i=0; i<N; i++) {
     inptxt.getline(line,BUFSIZ,'\n');
     tokenN = parseWords(line,words,3);
     if(tokenN != 2) {
-      std::cerr << "Wrong input format." << std::endl;
-      exit(1);
+			exit_error(IRSTLM_ERROR_DATA,"Wrong input format");
     }
     w[i] = (float) atof(words[0]);
     lmf[i] = words[1];
@@ -229,8 +230,7 @@ int main(int argc, char **argv)
 
   //Learning mixture weights
   if (learn) {
-
-    std::vector<float> p[N]; //LM probabilities
+    std::vector<float> *p = new std::vector<float>[N]; //LM probabilities
     float c[N]; //expected counts
     float den,norm; //inner denominator, normalization term
     float variation=1.0; // global variation between new old params
@@ -246,8 +246,7 @@ int main(int argc, char **argv)
       if(dev.eof())
         break;
       if(dev.fail()) {
-        std::cerr << "Problem reading input file " << seval << std::endl;
-        exit(1);
+				exit_error(IRSTLM_ERROR_IO,"Problem reading input file");
       }
       std::istringstream lstream(line);
       if(line.substr(0, 29) == "###interpolate-lm:replace-lm ") {
@@ -256,6 +255,7 @@ int main(int argc, char **argv)
         lstream >> token >> id >> newlm;
         if(id <= 0 || id > N) {
           std::cerr << "LM id out of range." << std::endl;
+	  delete[] p;
           return 1;
         }
         id--; // count from 0 now
@@ -313,11 +313,11 @@ int main(int argc, char **argv)
 
     //Saving results
     std::cerr << "Saving in " << outfile << "..." << std::endl;
-    //saving result
     std::fstream outtxt(outfile.c_str(),std::ios::out);
     outtxt << "LMINTERPOLATION " << N << "\n";
     for (int i=0; i<N; i++) outtxt << w[i] << " " << lmf[i] << "\n";
     outtxt.close();
+    delete[] p;
   }
 
   for(int i = 0; i < N; i++)
@@ -390,6 +390,7 @@ int main(int argc, char **argv)
 
       double bow;
       int bol=0;
+      ngram_state_t msidx;
       char *msp;
       unsigned int statesize;
 
@@ -415,7 +416,8 @@ int main(int argc, char **argv)
 
             ngram ong(lmt[i]->getDict());
             ong.trans(ng);
-            logpr = lmt[i]->clprob(ong,&bow,&bol,&msp,&statesize); //actual prob of the interpolation
+//            logpr = lmt[i]->clprob(ong,&bow,&bol,&msp,&statesize); //actual prob of the interpolation
+            logpr = lmt[i]->clprob(ong,&bow,&bol,&msidx,&msp,&statesize); //actual prob of the interpolation
             //logpr = lmt[i]->clprob(ong,&bow,&bol); //LM log-prob
 
             Pr+=w[i] * pow(10.0,logpr); //actual prob of the interpolation
@@ -519,8 +521,9 @@ int main(int argc, char **argv)
         for (i=0; i<N; i++) {
           ngram ong(lmt[i]->getDict());
           ong.trans(ng);
-          logpr = lmt[i]->clprob(ong,&bow,&bol,NULL,&statesize); //LM log-prob (using caches if available)
-
+//          logpr = lmt[i]->clprob(ong,&bow,&bol,NULL,&statesize); //LM log-prob (using caches if available)
+          logpr = lmt[i]->clprob(ong,&bow,&bol,NULL,NULL,&statesize); //LM log-prob (using caches if available)
+					
           Pr+=w[i] * pow(10.0,logpr); //actual prob of the interpolation
           std::cout << "lm " << i << ":" << " logpr: " << logpr << " weight: " << w[i] << std::endl;
           if (maxbol<bol) maxbol=bol;
@@ -550,10 +553,8 @@ int main(int argc, char **argv)
 
 lmContainer* load_lm(std::string file,int requiredMaxlev,int dub,int memmap, float nlf, float dlf)
 {
-  lmContainer* lmt=NULL;
-
-  lmt = lmt->CreateLanguageModel(file,nlf,dlf);
-
+  lmContainer* lmt = lmContainer::CreateLanguageModel(file,nlf,dlf);
+	
   lmt->setMaxLoadedLevel(requiredMaxlev);
 
   lmt->load(file,memmap);
