@@ -20,8 +20,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 
 ******************************************************************************/
 
-using namespace std;
-
+#include <sstream>
+#include "util.h"
 #include "mfstream.h"
 #include "math.h"
 #include "mempool.h"
@@ -29,6 +29,175 @@ using namespace std;
 #include "dictionary.h"
 #include "n_gram.h"
 #include "ngramtable.h"
+#include "crc.h"
+
+using namespace std;
+
+tabletype::tabletype(TABLETYPE tt,int codesize) {
+
+    if (codesize<=4 && codesize>0)
+      CODESIZE=codesize;
+    else {
+			exit_error(IRSTLM_ERROR_DATA,"ngramtable wrong codesize");
+    }
+
+    code_range[1]=255;
+    code_range[2]=65535;
+    code_range[3]=16777214;
+    code_range[4]=2147483640;
+    code_range[6]=140737488360000LL; //stay below true limit
+//	code_range[6]=281474977000000LL; //stay below true limit
+
+    //information which is useful to initialize
+    //LEAFPROB tables
+    L_FREQ_SIZE=FREQ1;
+
+    WORD_OFFS  =0;
+    MSUCC_OFFS =CODESIZE;
+    MTAB_OFFS  =MSUCC_OFFS+CODESIZE;
+    FLAGS_OFFS =MTAB_OFFS+PTRSIZE;
+
+    switch (tt) {
+
+    case COUNT:
+      SUCC1_OFFS =0;
+      SUCC2_OFFS =0;
+      BOFF_OFFS  =0;
+      I_FREQ_OFFS=FLAGS_OFFS+CHARSIZE;
+      I_FREQ_NUM=1;
+      L_FREQ_NUM=1;
+
+      ttype=tt;
+      break;
+
+    case FULL:
+    case IMPROVEDKNESERNEY_B:
+    case IMPROVEDSHIFTBETA_B:
+      SUCC1_OFFS =FLAGS_OFFS+CHARSIZE;
+      SUCC2_OFFS =SUCC1_OFFS+CODESIZE;
+      BOFF_OFFS  =SUCC2_OFFS+CODESIZE;
+      I_FREQ_OFFS=BOFF_OFFS+INTSIZE;
+      L_FREQ_OFFS=CODESIZE;
+      I_FREQ_NUM=2;
+      L_FREQ_NUM=1;
+
+      ttype=tt;
+      break;
+
+    case IMPROVEDKNESERNEY_I:
+    case IMPROVEDSHIFTBETA_I:
+      SUCC1_OFFS =FLAGS_OFFS+CHARSIZE;
+      SUCC2_OFFS =SUCC1_OFFS+CODESIZE;
+      BOFF_OFFS  =0;
+      I_FREQ_OFFS=SUCC2_OFFS+CODESIZE;
+      L_FREQ_OFFS=CODESIZE;
+      I_FREQ_NUM=2;
+      L_FREQ_NUM=1;
+
+      ttype=tt;
+      break;
+
+    case SIMPLE_I:
+      SUCC1_OFFS = 0;
+      SUCC2_OFFS = 0;
+      BOFF_OFFS  = 0;
+      I_FREQ_OFFS= FLAGS_OFFS+CHARSIZE;
+      L_FREQ_OFFS=CODESIZE;
+      I_FREQ_NUM=1;
+      L_FREQ_NUM=1;
+
+      ttype=tt;
+      break;
+
+    case SIMPLE_B:
+      SUCC1_OFFS  = 0;
+      SUCC2_OFFS  = 0;
+      BOFF_OFFS   = FLAGS_OFFS+CHARSIZE;
+      I_FREQ_OFFS = BOFF_OFFS+INTSIZE;
+      L_FREQ_OFFS = CODESIZE;
+      I_FREQ_NUM  = 1;
+      L_FREQ_NUM  = 1;
+
+      ttype=tt;
+      break;
+				
+		case KNESERNEY_I:
+		case SHIFTBETA_I:
+      SUCC1_OFFS = FLAGS_OFFS+CHARSIZE;
+      SUCC2_OFFS = 0;
+      BOFF_OFFS  = 0;
+      I_FREQ_OFFS= SUCC1_OFFS+CODESIZE;
+      L_FREQ_OFFS=CODESIZE;
+      I_FREQ_NUM=1;
+      L_FREQ_NUM=1;
+
+      ttype=tt;
+      break;
+
+		case KNESERNEY_B:
+    case SHIFTBETA_B:
+      SUCC1_OFFS  = FLAGS_OFFS+CHARSIZE;
+      SUCC2_OFFS  = 0;
+      BOFF_OFFS   = SUCC1_OFFS+CODESIZE;
+      I_FREQ_OFFS = BOFF_OFFS+INTSIZE;
+      L_FREQ_OFFS = CODESIZE;
+      I_FREQ_NUM  = 1;
+      L_FREQ_NUM  = 1;
+
+      ttype=tt;
+      break;
+
+    case LEAFPROB:
+    case FLEAFPROB:
+      SUCC1_OFFS  = 0;
+      SUCC2_OFFS  = 0;
+      BOFF_OFFS   = 0;
+      I_FREQ_OFFS = FLAGS_OFFS+CHARSIZE;
+      I_FREQ_NUM  = 0;
+      L_FREQ_NUM  = 1;
+
+      ttype=tt;
+      break;
+
+    case LEAFPROB2:
+      SUCC1_OFFS =0;
+      SUCC2_OFFS =0;
+      BOFF_OFFS  =0;
+      I_FREQ_OFFS=FLAGS_OFFS+CHARSIZE;
+      I_FREQ_NUM=0;
+      L_FREQ_NUM=2;
+
+      ttype=LEAFPROB;
+      break;
+
+    case LEAFPROB3:
+      SUCC1_OFFS =0;
+      SUCC2_OFFS =0;
+      BOFF_OFFS  =0;
+      I_FREQ_OFFS=FLAGS_OFFS+CHARSIZE;
+      I_FREQ_NUM=0;
+      L_FREQ_NUM=3;
+
+      ttype=LEAFPROB;
+      break;
+
+    case LEAFPROB4:
+      SUCC1_OFFS =0;
+      SUCC2_OFFS =0;
+      BOFF_OFFS  =0;
+      I_FREQ_OFFS=FLAGS_OFFS+CHARSIZE;
+      I_FREQ_NUM=0;
+      L_FREQ_NUM=4;
+
+      ttype=LEAFPROB;
+      break;
+
+    default:
+      MY_ASSERT(tt==COUNT);
+    }
+
+    L_FREQ_OFFS=CODESIZE;
+};
 
 ngramtable::ngramtable(char* filename,int maxl,char* /* unused parameter: is */, 
 					   dictionary* extdict /* external dictionary */,char* filterdictfile,
@@ -56,7 +225,9 @@ ngramtable::ngramtable(char* filename,int maxl,char* /* unused parameter: is */,
       if (strcmp(info,"LM_")==0) {
         inp >> resolution;
         inp >> decay;
-        sprintf(info,"%s %d %f",info,resolution,decay);
+				char       info2[100];
+        sprintf(info2,"%s %d %f",info,resolution,decay);
+				strcpy(info, info2);
       } else { //default for old LM probs
         resolution=10000000;
         decay=0.9999;
@@ -71,14 +242,12 @@ ngramtable::ngramtable(char* filename,int maxl,char* /* unused parameter: is */,
   }
 
   if (!maxl) {
-    cerr << "ngramtable: ngram size must be specified\n";
-    exit(1);
+		exit_error(IRSTLM_ERROR_DATA,"ngramtable: ngram size must be specified");
   }
 
   //distant co-occurreces works for bigrams and trigrams
   if (dstco && (maxl!=2) && (maxl!=3)) {
-    cerr << "distant co-occurrences work with 2-gram and 3-gram!\n";
-    exit(1);
+		exit_error(IRSTLM_ERROR_DATA,"distant co-occurrences work with 2-gram and 3-gram");
   }
 
   maxlev=maxl;
@@ -96,8 +265,7 @@ ngramtable::ngramtable(char* filename,int maxl,char* /* unused parameter: is */,
   else if (maxlev==1)
     mtflags(tree,LNODE | FREQ4);
   else {
-    cerr << "ngramtable: wrong level setting\n";
-    exit(1);
+		exit_error(IRSTLM_ERROR_DATA,"ngramtable: wrong level setting");
   }
 
   word(tree,0); // dummy variable
@@ -111,8 +279,8 @@ ngramtable::ngramtable(char* filename,int maxl,char* /* unused parameter: is */,
   mem=new storage(256,10000);
 
   mentr=new long long[maxlev+1];
-  memory= new int[maxlev+1];
-  occupancy= new int[maxlev+1];
+  memory= new long long[maxlev+1];
+  occupancy= new long long[maxlev+1];
 
 //Book keeping of occupied memory
   mentr[0]=1;
@@ -141,8 +309,7 @@ ngramtable::ngramtable(char* filename,int maxl,char* /* unused parameter: is */,
 
   if ((strncmp(header,"ngram",5)==0) ||
       (strncmp(header,"NGRAM",5)==0)) {
-    cerr << "this ngram file format is no more supported!\n";
-    exit(1);
+		exit_error(IRSTLM_ERROR_DATA,"this ngram file format is no more supported");
   }
 
   if (strncmp(header,"nGrAm",5)==0)
@@ -166,40 +333,62 @@ ngramtable::ngramtable(char* filename,int maxl,char* /* unused parameter: is */,
   }
 }
 
-void ngramtable::savetxt(char *filename,int depth,int googleformat)
+void ngramtable::savetxt(char *filename,int depth,bool googleformat,bool hashvalue,int startfrom)
 {
-
-  if (depth>maxlev) {
-    cerr << "savetxt: wrong n-gram size\n";
-    exit(1);
-  }
-
-  depth=(depth>0?depth:maxlev);
-
-  card=mentr[depth];
-
-  ngram ng(dict);
-
-  if (googleformat)
+    char ngstring[10000];
+    
+    if (depth>maxlev) {
+        exit_error(IRSTLM_ERROR_DATA,"ngramtable::savetxt: wrong n-gram size");
+    }
+    
+    if (startfrom>0 && !googleformat) {
+        exit_error(IRSTLM_ERROR_DATA,
+                   "ngramtable::savetxt: multilevel output only allowed in googleformat");
+    }
+    
+    depth=(depth>0?depth:maxlev);
+    
+    card=mentr[depth];
+    
+    ngram ng(dict);
+    
+    if (googleformat)
     cerr << "savetxt in Google format: nGrAm " <<  depth << " " << card << " " << info << "\n";
-  else
+    else
     cerr << "savetxt: nGrAm " <<  depth << " " << card << " " << info << "\n";
-
-  mfstream out(filename,ios::out );
-
-  if (!googleformat)
-    out << "nGrAm " << depth << " " << card << " " << info << "\n";
-
-  if (!googleformat)
-    dict->save(out);
-
-  scan(ng,INIT,depth);
-
-  while(scan(ng,CONT,depth)) out << ng <<"\n";
-
-  cerr << "\n";
-
-  out.close();
+    
+    mfstream out(filename,ios::out );
+    
+    if (!googleformat){
+        out << "nGrAm " << depth << " " << card << " " << info << "\n";
+        dict->save(out);
+    }
+    
+    if (startfrom<=0 || startfrom > depth) startfrom=depth;
+    
+    for (int d=startfrom;d<=depth;d++){
+        scan(ng,INIT,d);
+        
+        while(scan(ng,CONT,d)){
+            
+            if (hashvalue){
+                strcpy(ngstring,ng.dict->decode(*ng.wordp(ng.size)));
+                for (int i=ng.size-1; i>0; i--){
+                    strcat(ngstring," ");
+                    strcat(ngstring,ng.dict->decode(*ng.wordp(i)));
+                }
+                out << ngstring << "\t" << ng.freq << "\t" << crc16_ccitt(ngstring,strlen(ngstring)) << "\n";
+            }
+            else
+            
+            out << ng << "\n";
+            
+            
+        }
+    }
+    cerr << "\n";
+    
+    out.close();
 }
 
 
@@ -328,8 +517,7 @@ void ngramtable::savebin(char *filename,int depth)
 {
 
   if (depth > maxlev) {
-    cerr << "savebin: wrong n-gram size\n";
-    exit(1);
+		exit_error(IRSTLM_ERROR_DATA,"ngramtable::savebin: wrong n-gram size");
   }
 
   depth=(depth>0?depth:maxlev);
@@ -461,8 +649,9 @@ void ngramtable::generate(char *filename, dictionary* extdict)
   int i,c=0;
 
   if (!inp) {
-    cerr << "cannot open " << filename << "\n";
-    exit(1);
+		std::stringstream ss_msg;
+		ss_msg << "cannot open " << filename;
+		exit_error(IRSTLM_ERROR_IO, ss_msg.str());
   }
 
   cerr << "load:";
@@ -525,34 +714,38 @@ void ngramtable::generate(char *filename, dictionary* extdict)
 void ngramtable::generate_hmask(char *filename,char* hmask,int inplen)
 {
   mfstream inp(filename,ios::in);
-  int i,c=0;
-  int selmask[MAX_NGRAM];
 
   if (!inp) {
-    cerr << "cannot open " << filename << "\n";
-    exit(1);
+		std::stringstream ss_msg;
+		ss_msg << "cannot open " << filename;
+		exit_error(IRSTLM_ERROR_IO, ss_msg.str());
   }
 
+  int selmask[MAX_NGRAM];
+  memset(selmask, 0, sizeof(int)*MAX_NGRAM);
+
   //parse hmask
-  i=0;
-  selmask[i++]=1;
-  for (c=0; c< (int)strlen(hmask); c++) {
+  selmask[0]=1;
+  int i=1;
+  for (size_t c=0; c<strlen(hmask); c++) {
     cerr << hmask[c] << "\n";
-    if (hmask[c] == '1')
-      selmask[i++]=c+2;
+    if (hmask[c] == '1'){
+      selmask[i]=c+2;
+			i++;
+		}
   }
   if (i!= maxlev) {
-    cerr << "wrong mask: 1 bits=" << i << " maxlev=" << maxlev << "\n";
-    exit(1);
+		std::stringstream ss_msg;
+		ss_msg << "wrong mask: 1 bits=" << i << " maxlev=" << maxlev;
+		exit_error(IRSTLM_ERROR_DATA, ss_msg.str());
   }
 
-  //  for (i=0;i<maxlev;i++)
-  //cerr << "selmask " << i << "-> " << selmask[i] << "\n";
   cerr << "load:";
 
   ngram ng(dict);
   ngram ng2(dict);
   dict->incflag(1);
+	long c=0;
   while (inp >> ng) {
 
     if (inplen && ng.size<inplen) continue;
@@ -561,8 +754,8 @@ void ngramtable::generate_hmask(char *filename,char* hmask,int inplen)
     ng.size=0;    //reset  ng
 
     if (ng2.size >= selmask[maxlev-1]) {
-      for (i=0; i<maxlev; i++)
-        *ng2.wordp(i+1)=*ng2.wordp(selmask[i]);
+      for (int j=0; j<maxlev; j++)
+        *ng2.wordp(j+1)=*ng2.wordp(selmask[i]);
 
       //cout << ng2 << "size:" << ng2.size << "\n";
       check_dictsize_bound();
@@ -593,15 +786,17 @@ void ngramtable::generate_dstco(char *filename,int dstco)
   int c=0;
 
   if (!inp) {
-    cerr << "cannot open " << filename << "\n";
-    exit(1);
+		std::stringstream ss_msg;
+		ss_msg << "cannot open " << filename;
+		exit_error(IRSTLM_ERROR_IO, ss_msg.str());
   }
 
   cerr << "load distant co-occurrences:";
   if (dstco>MAX_NGRAM) {
-    cerr << "window size (" << dstco << ") exceeds MAXNGRAM\n";
     inp.close();
-    exit (1);
+		std::stringstream ss_msg;
+		ss_msg << "window size (" << dstco << ") exceeds MAXNGRAM";
+		exit_error(IRSTLM_ERROR_DATA, ss_msg.str());
   }
 
   ngram ng(dict);
@@ -685,9 +880,7 @@ void ngramtable::augment(ngramtable* ngt)
 {
 
   if (ngt->maxlev != maxlev) {
-    cerr << "ngt augmentation is not possible "
-         << "due to table incompatibility!";
-    exit(1);
+		exit_error(IRSTLM_ERROR_DATA,"ngramtable::augment augmentation is not possible due to table incompatibility");
   }
 
   if (ngt->dict->oovcode()!=-1)
@@ -1017,8 +1210,7 @@ char **ngramtable::grow(table *tb,NODETYPE ndt,int lev,
       else if (oldndt & FREQ4)
         oldsz=inodesize(4);
       else {
-        cerr << "funzione non prevista\n";
-        exit(1);
+				exit_error(IRSTLM_ERROR_DATA,"ngramtable::grow functionality not available");
       }
     } else if (ndt & LNODE) {
       if (oldndt & FREQ1)
@@ -1030,8 +1222,7 @@ char **ngramtable::grow(table *tb,NODETYPE ndt,int lev,
       else if (oldndt & FREQ4)
         oldsz=lnodesize(4);
       else {
-        cerr << "funzione non prevista\n";
-        exit(1);
+				exit_error(IRSTLM_ERROR_DATA,"ngramtable::grow functionality not available");
       }
     }
 
@@ -1206,11 +1397,10 @@ int ngramtable::get(ngram& ng,int n,int lev)
   char *found;
   NODETYPE ndt;
 
-  assert(lev <= n && lev <= maxlev && ng.size >= n);
+  MY_ASSERT(lev <= n && lev <= maxlev && ng.size >= n);
 
   if ((I_FREQ_NUM==0) && (lev < maxlev)) {
-    cerr << "get: for this type of table ngram cannot be smaller than table size\n";
-    exit(1);
+		exit_error(IRSTLM_ERROR_DATA,"ngramtable::get for this type of table ngram cannot be smaller than table size");
   }
 
 
@@ -1260,11 +1450,10 @@ int ngramtable::get(ngram& ng,int n,int lev)
 int ngramtable::scan(node nd,NODETYPE /* unused parameter: ndt */,int lev,ngram& ng,ACTION action,int maxl)
 {
 
-  assert(lev<=maxlev);
+  MY_ASSERT(lev<=maxlev);
 
   if ((I_FREQ_NUM==0) && (maxl < maxlev)) {
-    cerr << "scan: ngram cannot be smaller than LEAFPROB table\n";
-    exit(1);
+		exit_error(IRSTLM_ERROR_MODEL,"ngramtable::scan ngram cannot be smaller than LEAFPROB table");
   }
 
 
@@ -1366,8 +1555,8 @@ ngramtable::~ngramtable()
 
 void ngramtable::stat(int level)
 {
-  int totmem=0;
-  int totwaste=0;
+  long long totmem=0;
+  long long totwaste=0;
   float mega=1024 * 1024;
 
   cout.precision(2);
@@ -1402,7 +1591,7 @@ double ngramtable::prob(ngram ong)
   if (ong.size==0) return 0.0;
   if (ong.size>maxlev) ong.size=maxlev;
 
-  assert(tbtype()==LEAFPROB && ong.size<=maxlev);
+  MY_ASSERT(tbtype()==LEAFPROB && ong.size<=maxlev);
 
   ngram ng(dict);
   ng.trans(ong);
@@ -1445,27 +1634,237 @@ double ngramtable::prob(ngram ong)
 bool ngramtable::check_dictsize_bound()
 {
   if (dict->size() >= code_range[CODESIZE]) {
-    cerr
-        << "dictionary size overflows code range "
-        << code_range[CODESIZE] << "\n";
-    exit(1);
+		std::stringstream ss_msg;
+		ss_msg << "dictionary size overflows code range " << code_range[CODESIZE];
+		exit_error(IRSTLM_ERROR_MODEL, ss_msg.str());
   }
   return true;
 }
 
+int ngramtable::update(ngram ng) {
+
+    if (!get(ng,ng.size,ng.size)) {
+			std::stringstream ss_msg;
+			ss_msg << "cannot find " << ng;
+			exit_error(IRSTLM_ERROR_MODEL, ss_msg.str());
+    }
+
+    freq(ng.link,ng.pinfo,ng.freq);
+
+    return 1;
+}
+
+void ngramtable::resetngramtable() {
+    //clean up all memory and restart from an empty table
+
+    freetree(); //clean memory pool
+    memset(tree,0,inodesize(6)); //reset tree
+    //1-gram table initial flags
+
+    if (maxlev>1) mtflags(tree,INODE | FREQ4);
+    else if (maxlev==1) mtflags(tree,LNODE | FREQ4);
+
+    word(tree,0);      //dummy word
+    msucc(tree,0);     // number of different n-grams
+    mtable(tree,NULL); // table of n-gram
+
+    for (int i=1; i<=maxlev; i++)
+      mentr[i]=memory[i]=occupancy[i]=0;
+
+}
+
+int ngramtable::putmem(char* ptr,int value,int offs,int size) {
+    MY_ASSERT(ptr!=NULL);
+    for (int i=0; i<size; i++)
+      ptr[offs+i]=(value >> (8 * i)) & 0xff;
+    return value;
+}
+
+int ngramtable::getmem(char* ptr,int* value,int offs,int size) {
+    MY_ASSERT(ptr!=NULL);
+    *value=ptr[offs] & 0xff;
+    for (int i=1; i<size; i++)
+      *value= *value | ( ( ptr[offs+i] & 0xff ) << (8 *i));
+    return *value;
+}
+
+long ngramtable::putmem(char* ptr,long long value,int offs,int size) {
+    MY_ASSERT(ptr!=NULL);
+    for (int i=0; i<size; i++)
+      ptr[offs+i]=(value >> (8 * i)) & 0xffLL;
+    return value;
+}
+
+long ngramtable::getmem(char* ptr,long long* value,int offs,int size) {
+    MY_ASSERT(ptr!=NULL);
+    *value=ptr[offs] & 0xff;
+    for (int i=1; i<size; i++)
+      *value= *value | ( ( ptr[offs+i] & 0xffLL ) << (8 *i));
+    return *value;
+}
+
+void ngramtable::tb2ngcpy(int* wordp,char* tablep,int n) {
+    for (int i=0; i<n; i++)
+      getmem(tablep,&wordp[i],i*CODESIZE,CODESIZE);
+}
+
+void ngramtable::ng2tbcpy(char* tablep,int* wordp,int n) {
+    for (int i=0; i<n; i++)
+      putmem(tablep,wordp[i],i*CODESIZE,CODESIZE);
+}
+
+int ngramtable::ngtbcmp(int* wordp,char* tablep,int n) {
+    int word;
+    for (int i=0; i<n; i++) {
+      getmem(tablep,&word,i*CODESIZE,CODESIZE);
+      if (wordp[i]!=word) return 1;
+    }
+    return 0;
+}
+
+int ngramtable::codecmp(char * a,char *b) {
+    register int i,result;
+    for (i=(CODESIZE-1); i>=0; i--) {
+      result=(unsigned char)a[i]-(unsigned char)b[i];
+      if(result) return result;
+    }
+    return 0;
+};
+
+long long ngramtable::freq(node nd,NODETYPE ndt,long long value) {
+    int offs=(ndt & LNODE)?L_FREQ_OFFS:I_FREQ_OFFS;
+
+    if (ndt & FREQ1)
+      putmem(nd,value,offs,1);
+    else if (ndt & FREQ2)
+      putmem(nd,value,offs,2);
+    else if (ndt & FREQ3)
+      putmem(nd,value,offs,3);
+    else if (ndt & FREQ4)
+      putmem(nd,value,offs,4);
+    else
+      putmem(nd,value,offs,6);
+    return value;
+}
+
+long long ngramtable::freq(node nd,NODETYPE ndt) {
+    int offs=(ndt & LNODE)?L_FREQ_OFFS:I_FREQ_OFFS;
+    long long value;
+
+    if (ndt & FREQ1)
+      getmem(nd,&value,offs,1);
+    else if (ndt & FREQ2)
+      getmem(nd,&value,offs,2);
+    else if (ndt & FREQ3)
+      getmem(nd,&value,offs,3);
+    else if (ndt & FREQ4)
+      getmem(nd,&value,offs,4);
+    else
+      getmem(nd,&value,offs,6);
+
+    return value;
+}
+
+
+long long ngramtable::setfreq(node nd,NODETYPE ndt,long long value,int index) {
+    int offs=(ndt & LNODE)?L_FREQ_OFFS:I_FREQ_OFFS;
+
+    if (ndt & FREQ1)
+      putmem(nd,value,offs+index * 1,1);
+    else if (ndt & FREQ2)
+      putmem(nd,value,offs+index * 2,2);
+    else if (ndt & FREQ3)
+      putmem(nd,value,offs+index * 3,3);
+    else if (ndt & FREQ4)
+      putmem(nd,value,offs+index * 4,4);
+    else
+      putmem(nd,value,offs+index * 6,6);
+
+    return value;
+}
+
+long long ngramtable::getfreq(node nd,NODETYPE ndt,int index) {
+    int offs=(ndt & LNODE)?L_FREQ_OFFS:I_FREQ_OFFS;
+
+    long long value;
+
+    if (ndt & FREQ1)
+      getmem(nd,&value,offs+ index * 1,1);
+    else if (ndt & FREQ2)
+      getmem(nd,&value,offs+ index * 2,2);
+    else if (ndt & FREQ3)
+      getmem(nd,&value,offs+ index * 3,3);
+    else if (ndt & FREQ4)
+      getmem(nd,&value,offs+ index * 4,4);
+    else
+      getmem(nd,&value,offs+ index * 6,6);
+
+    return value;
+}
+
+table ngramtable::mtable(node nd) {
+    char v[PTRSIZE];;
+    for (int i=0; i<PTRSIZE; i++)
+      v[i]=nd[MTAB_OFFS+i];
+
+    return *(table *)v;
+}
+
+table ngramtable::mtable(node nd,table value) {
+    char *v=(char *)&value;
+    for (int i=0; i<PTRSIZE; i++)
+      nd[MTAB_OFFS+i]=v[i];
+    return value;
+}
+
+int ngramtable::mtablesz(node nd) {
+    if (mtflags(nd) & LNODE) {
+      if (mtflags(nd) & FREQ1)
+        return lnodesize(1);
+      else if (mtflags(nd) & FREQ2)
+        return lnodesize(2);
+      else if (mtflags(nd) & FREQ3)
+        return lnodesize(3);
+      else if (mtflags(nd) & FREQ4)
+        return lnodesize(4);
+      else
+        return lnodesize(6);
+    } else if (mtflags(nd) & INODE) {
+      if (mtflags(nd) & FREQ1)
+        return inodesize(1);
+      else if (mtflags(nd) & FREQ2)
+        return inodesize(2);
+      else if (mtflags(nd) & FREQ3)
+        return inodesize(3);
+      else if (mtflags(nd) & FREQ4)
+        return inodesize(4);
+      else
+        return inodesize(6);
+    } else {
+			exit_error(IRSTLM_ERROR_DATA,"ngramtable::mtablesz node has wrong flags");
+    }
+	
+	return lnodesize(1); //this instruction is never reached
+}
+
+
 /*
  main(int argc, char** argv){
-	 dictionary d(argv[1]);
+         dictionary d(argv[1]);
 
-	 ngram ng(&d);
+         ngram ng(&d);
 
-	 cerr << "caricato dizionario da " << argv[1] << "\n";
+         cerr << "caricato dizionario da " << argv[1] << "\n";
 
-	 ngramtable t(&d,argv[2],1);
+         ngramtable t(&d,argv[2],1);
 
-	 t.stat(1);
-	 t.savetxt(argv[3]);
+         t.stat(1);
+         t.savetxt(argv[3]);
 
  }
- */
+*/
+
+
+
+
 
