@@ -21,8 +21,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 #include <math.h>
+#include <sstream>
+#include "util.h"
 #include "mfstream.h"
 #include "mempool.h"
 #include "htable.h"
@@ -34,7 +35,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 #include "interplm.h"
 #include "mdiadapt.h"
 #include "shiftlm.h"
-
+	
+namespace irstlm {
 //
 //Shiftone interpolated language model
 //
@@ -64,7 +66,7 @@ int shiftone::discount(ngram ng_,int size,double& fstar,double& lambda, int cv)
   ngram ng(dict);
   ng.trans(ng_);
 
-  // cout << "size :" << size << " " << ng <<"\n";
+  //cerr << "size:" << size << " ng:|" << ng <<"|\n";
 
   if (size > 1) {
 
@@ -134,8 +136,7 @@ shiftbeta::shiftbeta(char* ngtfile,int depth,int prunefreq,double b,TABLETYPE tt
     for (int l=lmsize(); l>1; l--)
       beta[l]=b;
   } else {
-    cerr << "shiftbeta: beta must be < 1.0 and > 0\n";
-    exit (1);
+		exit_error(IRSTLM_ERROR_DATA,"shiftbeta::shiftbeta beta must be < 1.0 and > 0");
   }
 
   prunethresh=prunefreq;
@@ -236,7 +237,6 @@ int shiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int cv)
 
 
       if (get(ng,size,size) && (!prunesingletons() || ng.freq >1 || size<3)) {
-
         cv=(cv>ng.freq)?ng.freq:cv;
 
         if (ng.freq>cv) {
@@ -258,7 +258,7 @@ int shiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int cv)
           lambda=beta[size]*((double)(history.succ-1)/ //e` sparito il successore
                              (double)(history.freq-cv));
 
-          if (size>=3 && prunesingletons()) //take into acccount single event pruning
+          if (size>=3 && prunesingletons()) //take into account single event pruning
             lambda+=(1.0-beta[size]) * (double)(succ1(history.link)-(cv==1 && ng.freq==1?1:0))
                     /(double)(history.freq-cv);
         }
@@ -296,13 +296,13 @@ int shiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int cv)
 }
 
 //
-//Modified Shiftbeta language model
+//Improved Kneser-Ney language model (previously ModifiedShiftBeta)
 //
 
-mshiftbeta::mshiftbeta(char* ngtfile,int depth,int prunefreq,TABLETYPE tt):
+improvedkneserney::improvedkneserney(char* ngtfile,int depth,int prunefreq,TABLETYPE tt):
   mdiadaptlm(ngtfile,depth,tt)
 {
-  cerr << "Creating LM with Modified ShiftBeta smoothing\n";
+  cerr << "Creating LM with Improved Kneser-Ney smoothing\n";
 
   prunethresh=prunefreq;
   cerr << "PruneThresh: " << prunethresh << "\n";
@@ -314,7 +314,7 @@ mshiftbeta::mshiftbeta(char* ngtfile,int depth,int prunefreq,TABLETYPE tt):
 };
 
 
-int mshiftbeta::train()
+int improvedkneserney::train()
 {
 	
 	trainunigr();
@@ -344,19 +344,16 @@ int mshiftbeta::train()
 			
 			//skip ngrams containing _OOV
 			if (l>1 && ng.containsWord(dict->OOV(),l)) {
-				//cerr << "skp ngram" << ng << "\n";
 				continue;
 			}
 			
 			//skip n-grams containing </s> in context
 			if (l>1 && ng.containsWord(dict->EoS(),l-1)) {
-				//cerr << "skp ngram" << ng << "\n";
 				continue;
 			}
 			
 			//skip 1-grams containing <s>
 			if (l==1 && ng.containsWord(dict->BoS(),l)) {
-				//cerr << "skp ngram" << ng << "\n";
 				continue;
 			}
 			
@@ -367,7 +364,6 @@ int mshiftbeta::train()
 			else if (ng.freq==3) n3++;
 			else if (ng.freq==4) n4++;
 			if (l==1 && ng.freq >=3) unover3++;
-			
 		}
 		
 		if (l==1) {
@@ -377,15 +373,16 @@ int mshiftbeta::train()
 		}
 		
 		if (n1 == 0 || n2 == 0 ||  n1 <= n2) {
-			cerr << "Error: lower order count-of-counts cannot be estimated properly\n";
-			cerr << "Hint: use another smoothing method with this corpus.\n";
-			exit(1);
+			std::stringstream ss_msg;
+			ss_msg << "Error: lower order count-of-counts cannot be estimated properly\n";
+			ss_msg << "Hint: use another smoothing method with this corpus.\n";
+			exit_error(IRSTLM_ERROR_DATA,ss_msg.str());
 		}
 		
 		double Y=(double)n1/(double)(n1 + 2 * n2);
 		beta[0][l] = Y; //equivalent to  1 - 2 * Y * n2 / n1
 		
-		if (n3 ==0 || n4 == 0 || n2 <= n3 || n3 <= n4 ){
+		if (n3 == 0 || n4 == 0 || n2 <= n3 || n3 <= n4 ){
 			cerr << "Warning: higher order count-of-counts cannot be estimated properly\n";
 			cerr << "Fixing this problem by resorting only on the lower order count-of-counts\n";
 			
@@ -424,12 +421,12 @@ int mshiftbeta::train()
 
 
 
-int mshiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int cv)
+int improvedkneserney::discount(ngram ng_,int size,double& fstar,double& lambda, int cv)
 {
   ngram ng(dict);
   ng.trans(ng_);
 
-  //cout << "size :" << size << " " << ng <<"\n";
+  //cerr << "size:" << size << " ng:|" << ng <<"|\n";
 
   if (size > 1) {
 
@@ -464,8 +461,8 @@ int mshiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int cv
                  (double)(history.freq-cv);
 
           if ((size>=3 && prunesingletons()) ||
-              (size==maxlevel() && prunetopsingletons()))
-            //correction
+              (size==maxlevel() && prunetopsingletons())) // correction due to frequency pruning
+
             lambda+=(double)(suc[0] * (1-beta[0][size])) / (double)(history.freq-cv);
 
         } else {
@@ -479,7 +476,7 @@ int mshiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int cv
                  (double)(history.freq-cv);
 
           if ((size>=3 && prunesingletons()) ||
-              (size==maxlevel() && prunetopsingletons())) //correction
+              (size==maxlevel() && prunetopsingletons())) // correction due to frequency pruning
             lambda+=(double)(suc[0] * (1-beta[0][size])) / (double)(history.freq-cv);
 
           ng.freq>=3?suc[2]++:suc[ng.freq-1]++; //resume successor stat
@@ -491,7 +488,7 @@ int mshiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int cv
                (double)(history.freq-cv);
 
         if ((size>=3 && prunesingletons()) ||
-            (size==maxlevel() && prunetopsingletons())) //correction
+            (size==maxlevel() && prunetopsingletons())) // correction due to frequency pruning
           lambda+=(double)(suc[0] * (1-beta[0][size])) / (double)(history.freq-cv);
 
       }
@@ -506,8 +503,8 @@ int mshiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int cv
         *ng.wordp(1)=dict->oovcode();
         if (get(ng,size,size)) {
           ng.freq=mfreq(ng,size);
-          if ((!prunesingletons() || mfreq(ng,size)>1 || size<3) &&
-              (!prunetopsingletons() || mfreq(ng,size)>1 || size<maxlevel())) {
+          if ((!prunesingletons() || ng.freq>1 || size<3) &&
+              (!prunetopsingletons() || ng.freq>1 || size<maxlevel())) {
             double b=(ng.freq>=3?beta[2][size]:beta[ng.freq-1][size]);
             lambda+=(double)(ng.freq - b)/(double)(history.freq-cv);
           }
@@ -519,43 +516,266 @@ int mshiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int cv
     }
   } else { // unigram case, no cross-validation
 
+		fstar=unigrIKN(ng);
     lambda=0.0;
-
-    int unigrtotfreq=(size<lmsize()?btotfreq():totfreq());
-
-	
-	
-    if (get(ng,size,size))
-      fstar=(double) mfreq(ng,size)/(double)unigrtotfreq;
-    else {
-			 cerr << "Missing probability for word: " << dict->decode(*ng.wordp(1)) << "\n";					
-			 exit(1);
-		 }
   }
 
   return 1;
 }
-
-
+	
+	double improvedkneserney::unigrIKN(ngram ng)
+	{ 
+		int unigrtotfreq=(lmsize()>1)?btotfreq():totfreq();
+		double fstar=0.0;
+		if (get(ng,1,1))
+			fstar=(double) mfreq(ng,1)/(double)unigrtotfreq;
+		else {
+			std::stringstream ss_msg;
+			ss_msg << "Missing probability for word: " << dict->decode(*ng.wordp(1));
+			exit_error(IRSTLM_ERROR_DATA,ss_msg.str());
+		}
+		return fstar;
+	}
+	
+	//
+	//Improved Shiftbeta language model (similar to Improved Kneser-Ney without corrected counts)
+	//
+	
+	improvedshiftbeta::improvedshiftbeta(char* ngtfile,int depth,int prunefreq,TABLETYPE tt):
+  mdiadaptlm(ngtfile,depth,tt)
+	{
+		cerr << "Creating LM with Improved ShiftBeta smoothing\n";
+		
+		prunethresh=prunefreq;
+		cerr << "PruneThresh: " << prunethresh << "\n";
+		
+		beta[1][0]=0.0;
+		beta[1][1]=0.0;
+		beta[1][2]=0.0;
+		
+	};
+	
+	
+	int improvedshiftbeta::train()
+	{
+		
+		trainunigr();
+		
+		gensuccstat();
+		
+		ngram ng(dict);
+		int n1,n2,n3,n4;
+		int unover3=0;
+		
+		oovsum=0;
+		
+		for (int l=1; l<=lmsize(); l++) {
+			
+			cerr << "level " << l << "\n";
+			
+			cerr << "computing statistics\n";
+			
+			n1=0;
+			n2=0;
+			n3=0,n4=0;
+			
+			scan(ng,INIT,l);
+			
+			while(scan(ng,CONT,l)) {
+				
+				//skip ngrams containing _OOV
+				if (l>1 && ng.containsWord(dict->OOV(),l)) {
+					continue;
+				}
+				
+				//skip n-grams containing </s> in context
+				if (l>1 && ng.containsWord(dict->EoS(),l-1)) {
+					continue;
+				}
+				
+				//skip 1-grams containing <s>
+				if (l==1 && ng.containsWord(dict->BoS(),l)) {
+					continue;
+				}
+				
+				ng.freq=mfreq(ng,l);
+				
+				if (ng.freq==1) n1++;
+				else if (ng.freq==2) n2++;
+				else if (ng.freq==3) n3++;
+				else if (ng.freq==4) n4++;
+				if (l==1 && ng.freq >=3) unover3++;
+			}
+			
+			if (l==1) {
+				cerr << " n1: " << n1 << " n2: " << n2 << " n3: " << n3 << " n4: " << n4 << " unover3: " << unover3 << "\n";
+			} else {
+				cerr << " n1: " << n1 << " n2: " << n2 << " n3: " << n3 << " n4: " << n4 << "\n";
+			}
+			
+			if (n1 == 0 || n2 == 0 ||  n1 <= n2) {
+				std::stringstream ss_msg;
+				ss_msg << "Error: lower order count-of-counts cannot be estimated properly\n";
+				ss_msg << "Hint: use another smoothing method with this corpus.\n";
+				exit_error(IRSTLM_ERROR_DATA,ss_msg.str());
+			}
+			
+			double Y=(double)n1/(double)(n1 + 2 * n2);
+			beta[0][l] = Y; //equivalent to  1 - 2 * Y * n2 / n1
+			
+			if (n3 == 0 || n4 == 0 || n2 <= n3 || n3 <= n4 ){
+				cerr << "Warning: higher order count-of-counts cannot be estimated properly\n";
+				cerr << "Fixing this problem by resorting only on the lower order count-of-counts\n";
+				
+				beta[1][l] = Y;
+				beta[2][l] = Y;			
+			}
+			else{ 	  
+				beta[1][l] = 2 - 3 * Y * n3 / n2; 
+				beta[2][l] = 3 - 4 * Y * n4 / n3;  
+			}
+			
+			if (beta[1][l] < 0){
+				cerr << "Warning: discount coefficient is negative \n";
+				cerr << "Fixing this problem by setting beta to 0 \n";			
+				beta[1][l] = 0;
+				
+			}		
+			
+			
+			if (beta[2][l] < 0){
+				cerr << "Warning: discount coefficient is negative \n";
+				cerr << "Fixing this problem by setting beta to 0 \n";			
+				beta[2][l] = 0;
+				
+			}
+			
+			
+			if (l==1)
+				oovsum=beta[0][l] * (double) n1 + beta[1][l] * (double)n2 + beta[2][l] * (double)unover3;
+			
+			cerr << beta[0][l] << " " << beta[1][l] << " " << beta[2][l] << "\n";
+		}
+		
+		return 1;
+	};
+	
+	
+	
+	int improvedshiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int cv)
+	{
+		ngram ng(dict);
+		ng.trans(ng_);
+		
+		//cerr << "size:" << size << " ng:|" << ng <<"|\n";
+		
+		if (size > 1) {
+			
+			ngram history=ng;
+			
+			//singleton pruning only on real counts!!
+			if (ng.ckhisto(size) && get(history,size,size-1) && (history.freq > cv) &&
+					((size < 3) || ((history.freq-cv) > prunethresh ))) { // no history pruning with corrected counts!
+				
+				int suc[3];
+				suc[0]=succ1(history.link);
+				suc[1]=succ2(history.link);
+				suc[2]=history.succ-suc[0]-suc[1];
+				
+				
+				if (get(ng,size,size) &&
+						(!prunesingletons() || mfreq(ng,size)>1 || size<3) &&
+						(!prunetopsingletons() || mfreq(ng,size)>1 || size<maxlevel())) {
+					
+					ng.freq=mfreq(ng,size);
+					
+					cv=(cv>ng.freq)?ng.freq:cv;
+					
+					if (ng.freq>cv) {
+						
+						double b=(ng.freq-cv>=3?beta[2][size]:beta[ng.freq-cv-1][size]);
+						
+						fstar=(double)((double)(ng.freq - cv) - b)/(double)(history.freq-cv);
+						
+						lambda=(beta[0][size] * suc[0] + beta[1][size] * suc[1] + beta[2][size] * suc[2])
+						/
+						(double)(history.freq-cv);
+						
+						if ((size>=3 && prunesingletons()) ||
+								(size==maxlevel() && prunetopsingletons())) // correction due to frequency pruning
+							
+							lambda+=(double)(suc[0] * (1-beta[0][size])) / (double)(history.freq-cv);
+						
+					} else {
+						// ng.freq==cv
+						
+						ng.freq>=3?suc[2]--:suc[ng.freq-1]--; //update successor stat
+						
+						fstar=0.0;
+						lambda=(beta[0][size] * suc[0] + beta[1][size] * suc[1] + beta[2][size] * suc[2])
+						/
+						(double)(history.freq-cv);
+						
+						if ((size>=3 && prunesingletons()) ||
+								(size==maxlevel() && prunetopsingletons())) // correction due to frequency pruning
+							lambda+=(double)(suc[0] * (1-beta[0][size])) / (double)(history.freq-cv);
+						
+						ng.freq>=3?suc[2]++:suc[ng.freq-1]++; //resume successor stat
+					}
+				} else {
+					fstar=0.0;
+					lambda=(beta[0][size] * suc[0] + beta[1][size] * suc[1] + beta[2][size] * suc[2])
+					/
+					(double)(history.freq-cv);
+					
+					if ((size>=3 && prunesingletons()) ||
+							(size==maxlevel() && prunetopsingletons())) // correction due to frequency pruning
+						lambda+=(double)(suc[0] * (1-beta[0][size])) / (double)(history.freq-cv);
+					
+				}
+				
+				//cerr << "ngram :" << ng << "\n";
+				
+				
+				if (*ng.wordp(1)==dict->oovcode()) {
+					lambda+=fstar;
+					fstar=0.0;
+				} else {
+					*ng.wordp(1)=dict->oovcode();
+					if (get(ng,size,size)) {
+						ng.freq=mfreq(ng,size);
+						if ((!prunesingletons() || ng.freq>1 || size<3) &&
+								(!prunetopsingletons() || ng.freq>1 || size<maxlevel())) {
+							double b=(ng.freq>=3?beta[2][size]:beta[ng.freq-1][size]);
+							lambda+=(double)(ng.freq - b)/(double)(history.freq-cv);
+						}
+					}
+				}
+			} else {
+				fstar=0;
+				lambda=1;
+			}
+		} else { // unigram case, no cross-validation
+			fstar=unigr(ng);
+			lambda=0;
+		}
+		
+		return 1;
+	}
+	
 //Symmetric Shiftbeta
 int symshiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int /* unused parameter: cv */)
 {
   ngram ng(dict);
   ng.trans(ng_);
 
-  //  cout << "size :" << size << " " << ng <<"\n";
+  //cerr << "size:" << size << " ng:|" << ng <<"|\n";
 
   // Pr(x/y)= max{(c([x,y])-beta)/(N Pr(y)),0} + lambda Pr(x)
   // lambda=#bigrams/N
 
-  assert(size<=2); // only works with bigrams //
+  MY_ASSERT(size<=2); // only works with bigrams //
 
-  if (size == 3) {
-
-    ngram history=ng;
-
-
-  }
   if (size == 2) {
 
     //compute unigram probability of denominator
@@ -585,6 +805,7 @@ int symshiftbeta::discount(ngram ng_,int size,double& fstar,double& lambda, int
   return 1;
 }
 
+}//namespace irstlm
 
 
 /*
